/*
 * This Java source file was generated by the Gradle 'init' task.
 */
package lightcontainer;

import lightcontainer.domains.server.MulticastServerListener;
import lightcontainer.domains.server.UnicastServerListener;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.protocol.rules.reader.*;
import lightcontainer.protocol.rules.writer.*;
import lightcontainer.repository.ClientHandlerRepository;
import lightcontainer.repository.FileFrontEnd;
import lightcontainer.repository.ProtocolRepositoryImpl;
import lightcontainer.repository.StoreProcessorRepository;
import lightcontainer.storage.AppData;
import lightcontainer.storage.JsonAdapter;
import lightcontainer.storage.Repository;

import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

public class App {


    public static void main(String[] args) {
        setupVM();
        Repository repositoryStorage = prepareStorage();

        // Create all repository
        ClientHandlerRepository clientRep = new ClientHandlerRepository();
        StoreProcessorRepository storeRep = new StoreProcessorRepository();
        ProtocolRepository protocolRep = new ProtocolRepositoryImpl();
        protocolRep.addReader(new HelloRule());
        protocolRep.addReader(new SigninRule(protocolRep));
        protocolRep.addReader(new SignupRule(protocolRep));
        protocolRep.addReader(new SignoutRule());
        protocolRep.addReader(new FilelistRule(protocolRep));
        protocolRep.addReader(new SavefileRule(protocolRep, repositoryStorage.getStoragePath()));
        protocolRep.addReader(new SendOkRule(protocolRep));

        protocolRep.addWriter(new SignOkRule());
        protocolRep.addWriter(new SignErrorRule());
        protocolRep.addWriter(new FilesRule());
        protocolRep.addWriter(new SaveFileOkRule());
        protocolRep.addWriter(new SaveFileErrorRule());
        protocolRep.addWriter(new SendfileRule(repositoryStorage.getStoragePath()));

        FileFrontEnd ffe = new FileFrontEnd(clientRep, storeRep, protocolRep);
        new UnicastServerListener(ffe, clientRep, protocolRep, repositoryStorage, repositoryStorage.getUnicastPort());
        new MulticastServerListener(ffe, storeRep, protocolRep, repositoryStorage.getMulticastIp(), repositoryStorage.getMulticastPort());

        // close repo et client et server.

        // Thread.sleep(60000);

        // clientRep.close();
        // storeRep.close();
    }

    private static Repository prepareStorage() {
        AppData appData = AppData.getInstance();
        Repository repository = new Repository(
                Paths.get("src", "main", "resources", "appdata.json").toAbsolutePath().toString(),
                appData, new JsonAdapter()
        );

        repository.load();
        return repository;
    }

    private static void setupVM() {
        System.setProperty("javax.net.ssl.keyStore","../ffe.labo.swilabus.com.p12");
        System.setProperty("javax.net.ssl.keyStorePassword","labo2022");
        System.setProperty("https.protocols","TLSv1.3");
    }

}
